# Hunter's Say Sounds Addon
#  
# Install instructions:
#       1. Install Mattie's EventScripts 2.0 plugin:
#           http://mattie.info/cs/
#
#       2. Copy and upload this script to:
#           <gamedir>/addons/eventscripts/say_sounds/
#
#       3. Add the following line somewhere in autoexec.cfg:
#           es_load say_sounds
#
################################################################################
#import EventScripts
import es
#import Libraries
import services
import popuplib
import playerlib
import keyvalues
import settinglib
import gamethread
#import Psyco Compiler
import psyco
psyco.full()

# Addon Information
info = es.AddonInfo()
info.name = "Say Sounds"
info.version = "2.0.0"
info.author = "Hunter"
info.url = "http://addons.eventscripts.com/addons/user/289"
info.description = "Simple Say Sounds"
info.basename = "say_sounds"

hunter_say_sounds_ver   = info.version
hunter_say_sounds_text  = 'Hunters '+info.name+', www.sourceplugins.de, '+info.version+', ES 2.0.0.247+ (Python Version)'

# Server Variables
say_sounds_savetime     = es.ServerVar('say_sounds_savetime', '30', 'How much days should inactive users be stored, before their settings get deleted? ( default = 30 )')
say_sounds_soundload    = es.ServerVar('say_sounds_soundload', '1', 'Should the sounds be downloaded with EventScripts?')
say_sounds_default      = es.ServerVar('say_sounds_default', '1', 'Default settings for new connecting players')
say_sounds_limit        = es.ServerVar('say_sounds_limit', '2', 'How much sounds can normal (non-admin) players play each round?')
say_sounds_force        = es.ServerVar('say_sounds_force', '0', 'Should the admin sounds always play?')
say_sounds_show         = es.ServerVar('say_sounds_show', '1', 'Should the sound trigger be shown?')

# KeyValues Object
say_sounds_kv           = keyvalues.KeyValues(name='say_sounds')

# Popuplib Object
say_sounds_normal       = popuplib.easymenu('say_sounds_normal', None, None)
say_sounds_admin        = popuplib.easymenu('say_sounds_admin', None, None)

# Settinglib Object
say_sounds_setting      = settinglib.create('saysounds', 'Say Sounds Setting', 'toggle')

# Global Variables
say_sounds_players      = {}

def load():
    public = es.ServerVar('hu_so', info.version, info.name)
    public.makepublic()
    
    auth = services.use('auth')
    auth.registerCapability('admin_sounds', auth.ADMIN)
    
    say_sounds_kv.load(es.getAddonPath('say_sounds')+'/say_sounds.txt')
    for sound in say_sounds_kv['normal'].keys():
        say_sounds_normal.addoption(sound, sound)
        say_sounds_admin.addoption(sound, sound)
    for sound in say_sounds_kv['admin'].keys():
        say_sounds_admin.addoption(sound, sound)
        
    say_sounds_normal.menuselectfb = _select_sound
    say_sounds_normal.settitle('Say Sounds')
    say_sounds_normal.vguititle = 'Say Sounds'
    say_sounds_admin.menuselectfb = _select_sound
    say_sounds_admin.settitle('Say Sounds Admin')
    say_sounds_admin.vguititle = 'Say Sounds Admin'

    say_sounds_setting.addoption('normal', 'Player Sounds')
    say_sounds_setting.addoption('admin', 'Admin Sounds')
    say_sounds_setting.setdefault('normal', str(say_sounds_default))
    say_sounds_setting.setdefault('admin', str(say_sounds_default))
    say_sounds_setting.addsound('ui/buttonclick.wav')

    es.regsaycmd('!sounds', 'say_sounds/saycmd', 'Say Sounds Setting')
    es.regsaycmd('!soundlist', 'say_sounds/listcmd', 'Say Sounds List')
    es.addons.registerSayFilter(playsound)
    es.log(hunter_say_sounds_text)
    es.msg('#multi', '#green[SaySounds] #defaultLoaded')
    
def unload():
    es.addons.unregisterSayFilter(playsound)
    es.unregsaycmd('!sounds')
    es.unregsaycmd('!soundlist')
    es.msg('#multi', '#green[SaySounds] #defaultUnloaded')
    
def es_map_start(event_var):
    global say_sounds_players
    say_sounds_setting.clear(int(say_sounds_savetime)*86400)
    say_sounds_players = {}
    if int(say_sounds_soundload):
        for key in say_sounds_kv.keys():
            for sound in say_sounds_kv[key].keys():
                es.stringtable('downloadables', 'sound/'+say_sounds_kv[key][sound])

def es_client_command(event_var):
    if (str(event_var['command']) == '!hunter_say_sounds_ver') or (str(event_var['command']) == '!hunter_all_ver'):
        es.cexec(int(event_var['userid']), 'echo '+hunter_say_sounds_text)

def player_activate(event_var):
    say_sounds_setting.updateTime(int(event_var['userid']))
    gamethread.delayed(30, es.cexec, (int(event_var['userid']), 'echo '+hunter_say_sounds_text))
    gamethread.delayed(15, es.tell, (int(event_var['userid']), '#multi', '#green[SaySounds] #defaultSay \'!soundlist\' to see all sounds'))

def player_disconnect(event_var):
    say_sounds_setting.updateTime(int(event_var['userid']))
    if int(event_var['userid']) in say_sounds_players:
        del say_sounds_players[int(event_var['userid'])]

def player_hurt(event_var):
    userid = int(event_var['userid'])
    attackerid = int(event_var['attacker'])
    if not es.isbot(attackerid):
        if int(event_var['es_userteam']) == int(event_var['es_attackerteam']):
            if userid != attackerid:
                if say_sounds_setting.get('teammate', attackerid):
                    es.playsound(attackerid, 'player/hitsounds/hitteammate.wav', 1.0)
        elif int(event_var['hitgroup']) == 1:
            if say_sounds_setting.get('headshot', attackerid):
                es.playsound(attackerid, 'player/hitsounds/headshot.wav', 1.0)
        else:
            if say_sounds_setting.get('normal', attackerid):
                es.playsound(attackerid, 'player/hitsounds/hit.wav', 1.0)

def playsound(userid, text, teamonly):
    play = False
    auth = services.use('auth')
    if not userid in say_sounds_players:
        say_sounds_players[userid] = 0
    if auth.isUseridAuthorized(userid, 'admin_sounds'):
        for sound in say_sounds_kv['admin'].keys():
            if text.startswith(sound):
                play = finalizesound(userid, 'admin', sound)
        for sound in say_sounds_kv['normal'].keys():
            if text.startswith(sound):
                play = finalizesound(userid, 'normal', sound)
    elif say_sounds_players[userid] < int(say_sounds_limit):
        for sound in say_sounds_kv['normal'].keys():
            if text.startswith(sound):
                play = finalizesound(userid, 'normal', sound)
    if play and not int(say_sounds_show):
        return (0, None, None)
    else:
        return (userid, text, teamonly)

def finalizesound(userid, key, sound):
    auth = services.use('auth')
    es.msg('#multi', '#green[SaySounds] #lightgreen%s #defaultplayed sound #lightgreen%s' % (es.getplayername(userid), sound))
    for target in playerlib.getUseridList('#human'):
        if (key == 'normal' and say_sounds_setting.get('normal', target)) or (key == 'admin' and say_sounds_setting.get('admin', target)) or (int(say_sounds_force) and auth.isUseridAuthorized(userid, 'admin_sounds')):
            es.playsound(target, say_sounds_kv[key][sound], 1.0)
    return True

def saycmd():
    say_sounds_setting.send(int(es.getcmduserid()))

def listcmd():
    auth = services.use('auth')
    if auth.isUseridAuthorized(int(es.getcmduserid()), 'admin_sounds'):
        say_sounds_admin.send(int(es.getcmduserid()))
    else:
        say_sounds_normal.send(int(es.getcmduserid()))

def _select_sound(userid, choice, popupid):
    es.sexec(userid, 'say %s' % choice)
